//
//  ContentView.swift
//  words3
//
//  Created by Pete Keleher on 10/14/21.
//

import SwiftUI
import CoreData

struct ContentView: View {
    @Environment(\.managedObjectContext) private var viewContext
    
    @FetchRequest(
        sortDescriptors: [NSSortDescriptor(keyPath: \Word.spelling, ascending: true)],
        animation: .default)
    private var items: FetchedResults<Word>

    @State var rack = ""
    @State var pattern = ""

    struct WordType : Codable {
        var words: [String]
    }
    
    func filter() -> [FetchedResults<Word>.Element] {
        let rackCounts = LetterCounts(rack)
        var ret = [FetchedResults<Word>.Element]()

        for item in items {
            let candidate = item.spelling!
            let candidateCounts = LetterCounts(candidate)
            
            if candidateCounts < rackCounts || candidateCounts == rackCounts {
                ret.append(item)
            }
        }

        return ret
    }
    var body: some View {
        Form {
            Section {
                TextField("rack", text: $rack)
                    .textInputAutocapitalization(.never)
                    .disableAutocorrection(true)
                TextField("pattern", text: $pattern)
                    .textInputAutocapitalization(.never)
                    .disableAutocorrection(true)
                    .onSubmit {
                        do {
                            let regex = try NSRegularExpression(pattern: pattern, options: [])
                        } catch {
                            print("REGERR: \(error)")
                            return
                        }
                        let s2 = "%K matches \"" + pattern + "\""
                        print(s2)
                        items.nsPredicate = NSPredicate(format: s2, #keyPath(Word.spelling))
                    }
            }
            Section(header: Text("Words")) {
                List {
                    ForEach(filter()) { word in
                        Text(word.spelling!)
                    }
                }
            }
            Section {
                Button("Delete") {
                    for item in items {
                        viewContext.delete(item)
                    }
                    do {
                        try viewContext.save()
                    } catch { fatalError("bad") }
                }
            }
            Section {
                Button("Load") {
                    do {
                        let path = Bundle.main.path(forResource: "smallWords", ofType: "json")
                        let data = try Data(contentsOf: URL(fileURLWithPath: path!))
                        if let jsonresults = try? JSONDecoder().decode(WordType.self, from: data) {
                            for word in jsonresults.words {
                                let newword = Word(context: viewContext)
                                newword.spelling = word
                            }
                            try viewContext.save()
                        }
                    } catch {
                        fatalError("BAD: \(error)")
                    }
                }
            }
        }
    }
}



struct ContentView_Previews: PreviewProvider {
    static var previews: some View {
        ContentView().environment(\.managedObjectContext, PersistenceController.preview.container.viewContext)
    }
}
